@@ -17,16 +17,8 @@ module Spec
       Gem::Platform.new(platform)
     end
 
-    # Returns a number smaller than the size of the index. Useful for specs that
-    # need the API request limit to be reached for some reason.
-    def low_api_request_limit_for(gem_repo)
-      all_gems = Dir[gem_repo.join("gems/*.gem")]
-
-      all_gem_names = all_gems.map do |file|
-        File.basename(file, ".gem").match(/\A(?<gem_name>[^-]+)-.*\z/)[:gem_name]
-      end.uniq
-
-      (all_gem_names - ["bundler"]).size
+    def rake_version
+      "13.2.1"
     end
 
     def build_repo1
@@ -61,7 +53,7 @@ module Spec
 
         build_gem "rails", "2.3.2" do |s|
           s.executables = "rails"
-          s.add_dependency "rake",           "13.0.1"
+          s.add_dependency "rake",           rake_version
           s.add_dependency "actionpack",     "2.3.2"
           s.add_dependency "activerecord",   "2.3.2"
           s.add_dependency "actionmailer",   "2.3.2"
@@ -85,11 +77,11 @@ module Spec
           s.add_dependency "activesupport", ">= 2.0.0"
         end
 
-        build_gem "rspec", "1.2.7", :no_default => true do |s|
+        build_gem "rspec", "1.2.7", no_default: true do |s|
           s.write "lib/spec.rb", "SPEC = '1.2.7'"
         end
 
-        build_gem "rack-test", :no_default => true do |s|
+        build_gem "rack-test", no_default: true do |s|
           s.write "lib/rack/test.rb", "RACK_TEST = '1.0'"
         end
 
@@ -203,27 +195,25 @@ module Spec
       end
     end
 
-    def build_repo2(&blk)
+    def build_repo2(**kwargs, &blk)
       FileUtils.rm_rf gem_repo2
       FileUtils.cp_r gem_repo1, gem_repo2
-      update_repo2(&blk) if block_given?
+      update_repo2(**kwargs, &blk) if block_given?
     end
 
     # A repo that has no pre-installed gems included. (The caller completely
     # determines the contents with the block.)
-    def build_repo4(&blk)
+    def build_repo4(**kwargs, &blk)
       FileUtils.rm_rf gem_repo4
-      build_repo(gem_repo4, &blk)
+      build_repo(gem_repo4, **kwargs, &blk)
     end
 
     def update_repo4(&blk)
       update_repo(gem_repo4, &blk)
     end
 
-    def update_repo2
-      update_repo gem_repo2 do
-        yield if block_given?
-      end
+    def update_repo2(**kwargs, &blk)
+      update_repo(gem_repo2, **kwargs, &blk)
     end
 
     def build_security_repo
@@ -241,12 +231,12 @@ module Spec
       end
     end
 
-    def build_repo(path, &blk)
+    def build_repo(path, **kwargs, &blk)
       return if File.directory?(path)
 
       FileUtils.mkdir_p("#{path}/gems")
 
-      update_repo(path, &blk)
+      update_repo(path,**kwargs, &blk)
     end
 
     def check_test_gems!
@@ -263,7 +253,7 @@ module Spec
       end
     end
 
-    def update_repo(path)
+    def update_repo(path, build_compact_index: true)
       if path == gem_repo1 && caller.first.split(" ").last == "`build_repo`"
         raise "Updating gem_repo1 is unsupported -- use gem_repo2 instead"
       end
@@ -272,7 +262,12 @@ module Spec
       @_build_repo = File.basename(path)
       yield
       with_gem_path_as Path.base_system_gem_path do
-        gem_command :generate_index, :dir => path
+        Dir[Spec::Path.base_system_gem_path.join("gems/rubygems-generate_index*/lib")].first ||
+          raise("Could not find rubygems-generate_index lib directory in #{Spec::Path.base_system_gem_path}")
+
+        command = "generate_index"
+        command += " --no-compact" if !build_compact_index && gem_command(command + " --help").include?("--[no-]compact")
+        gem_command command, dir: path
       end
     ensure
       @_build_path = nil
@@ -298,14 +293,14 @@ module Spec
       end
     end
 
-    def build_dep(name, requirements = Gem::Requirement.default, type = :runtime)
-      Bundler::Dependency.new(name, :version => requirements)
-    end
-
     def build_lib(name, *args, &blk)
       build_with(LibBuilder, name, args, &blk)
     end
 
+    def build_bundler(*args, &blk)
+      build_with(BundlerBuilder, "bundler", args, &blk)
+    end
+
     def build_gem(name, *args, &blk)
       build_with(GemBuilder, name, args, &blk)
     end
@@ -411,6 +406,49 @@ module Spec
       alias_method :dep, :runtime
     end
 
+    class BundlerBuilder
+      attr_writer :required_ruby_version
+
+      def initialize(context, name, version)
+        raise "can only build bundler" unless name == "bundler"
+
+        @context = context
+        @version = version || Bundler::VERSION
+      end
+
+      def _build(options = {})
+        full_name = "bundler-#{@version}"
+        build_path = @context.tmp + full_name
+        bundler_path = build_path + "#{full_name}.gem"
+
+        FileUtils.mkdir_p build_path
+
+        @context.shipped_files.each do |shipped_file|
+          target_shipped_file = shipped_file
+          target_shipped_file = shipped_file.sub(/\Alibexec/, "exe") if @context.ruby_core?
+          target_shipped_file = build_path + target_shipped_file
+          target_shipped_dir = File.dirname(target_shipped_file)
+          FileUtils.mkdir_p target_shipped_dir unless File.directory?(target_shipped_dir)
+          FileUtils.cp shipped_file, target_shipped_file, preserve: true
+        end
+
+        @context.replace_version_file(@version, dir: build_path)
+        @context.replace_required_ruby_version(@required_ruby_version, dir: build_path) if @required_ruby_version
+
+        Spec::BuildMetadata.write_build_metadata(dir: build_path)
+
+        @context.gem_command "build #{@context.relative_gemspec}", dir: build_path
+
+        if block_given?
+          yield(bundler_path)
+        else
+          FileUtils.mv bundler_path, options[:path]
+        end
+      ensure
+        build_path.rmtree
+      end
+    end
+
     class LibBuilder
       def initialize(context, name, version)
         @context = context
@@ -456,8 +494,6 @@ module Spec
         write "ext/extconf.rb", <<-RUBY
           require "mkmf"
 
-          $extout = "$(topdir)/" + RbConfig::CONFIG["EXTOUT"]
-
           extension_name = "#{name}_c"
           if extra_lib_dir = with_config("ext-lib")
             # add extra libpath if --with-ext-lib is
@@ -539,7 +575,7 @@ module Spec
         default_branch = options[:default_branch] || "main"
         path = options[:path] || _default_path
         source = options[:source] || "git@#{path}"
-        super(options.merge(:path => path, :source => source))
+        super(options.merge(path: path, source: source))
         @context.git("config --global init.defaultBranch #{default_branch}", path)
         @context.git("init", path)
         @context.git("add *", path)
@@ -553,7 +589,7 @@ module Spec
     class GitBareBuilder < LibBuilder
       def _build(options)
         path = options[:path] || _default_path
-        super(options.merge(:path => path))
+        super(options.merge(path: path))
         @context.git("init --bare", path)
       end
     end
@@ -578,7 +614,7 @@ module Spec
         _default_files.keys.each do |path|
           _default_files[path] += "\n#{Builders.constantize(name)}_PREV_REF = '#{current_ref}'"
         end
-        super(options.merge(:path => libpath, :gemspec => update_gemspec, :source => source))
+        super(options.merge(path: libpath, gemspec: update_gemspec, source: source))
         @context.git("commit -am BUMP", libpath)
       end
     end
@@ -601,7 +637,7 @@ module Spec
     class GemBuilder < LibBuilder
       def _build(opts)
         lib_path = opts[:lib_path] || @context.tmp(".tmp/#{@spec.full_name}")
-        lib_path = super(opts.merge(:path => lib_path, :no_default => opts[:no_default]))
+        lib_path = super(opts.merge(path: lib_path, no_default: opts[:no_default]))
         destination = opts[:path] || _default_path
         FileUtils.mkdir_p(lib_path.join(destination))
 
@@ -610,16 +646,16 @@ module Spec
             Bundler.rubygems.build(@spec, opts[:skip_validation])
           end
         elsif opts[:skip_validation]
-          @context.gem_command "build --force #{@spec.name}", :dir => lib_path
+          @context.gem_command "build --force #{@spec.name}", dir: lib_path
         else
-          @context.gem_command "build #{@spec.name}", :dir => lib_path
+          @context.gem_command "build #{@spec.name}", dir: lib_path
         end
 
         gem_path = File.expand_path("#{@spec.full_name}.gem", lib_path)
         if opts[:to_system]
-          @context.system_gems gem_path, :default => opts[:default]
+          @context.system_gems gem_path, default: opts[:default]
         elsif opts[:to_bundle]
-          @context.system_gems gem_path, :path => @context.default_bundle_path
+          @context.system_gems gem_path, path: @context.default_bundle_path
         else
           FileUtils.mv(gem_path, destination)
         end
@@ -639,7 +675,7 @@ module Spec
       end
     end
 
-    TEST_CERT = <<-CERT.gsub(/^\s*/, "")
+    TEST_CERT = <<~CERT
       -----BEGIN CERTIFICATE-----
       MIIDMjCCAhqgAwIBAgIBATANBgkqhkiG9w0BAQUFADAnMQwwCgYDVQQDDAN5b3Ux
       FzAVBgoJkiaJk/IsZAEZFgdleGFtcGxlMB4XDTE1MDIwODAwMTIyM1oXDTQyMDYy
@@ -662,7 +698,7 @@ module Spec
       -----END CERTIFICATE-----
     CERT
 
-    TEST_PKEY = <<-PKEY.gsub(/^\s*/, "")
+    TEST_PKEY = <<~PKEY
       -----BEGIN RSA PRIVATE KEY-----
       MIIEowIBAAKCAQEA2W8V2k3jdzgMxL0mjTqbRruTdtDcdZDXKtiFkyLvsXUXvc2k
       GSdgcjMOS1CkafqGz/hAUlPibjM0QEXjtQuMdTmdMrmuORLeeIZhSO+HdkTNV6j3
